#!/usr/bin/env python3
# -*- coding:utf-8 -*-

import warnings
warnings.warn('Module ‘ckipparser’ is deprecated. Please import ‘ckipnlp.parser’ instead.', FutureWarning)
from ckipnlp.parser import *
from ckipnlp import __version__
